package communicationmethods;


import exceptions.networkexceptions.CreationAndEditException;
import exceptions.networkexceptions.ResourceNotFoundException;
import org.springframework.http.ResponseEntity;

import java.util.List;

/**
 * interface provides REST api
 */
public interface IServerMethods {
	/**
	 * testmethod, can be used to test communication
	 * @return ResponseEntity mit String Data
	 */
	ResponseEntity<String> test();

	/**
	 * creates a shape of type-template with all given parameters
	 *
	 * @param clientId=id of client
	 * @param type=shape
	 *@param canvasId=Layerid
	 * @param cx=x1-point - Default:Null
	 * @param cy=y1-point - Default:Null
	 * @param rx=radius for circle/x2 point for line/width - not required
	 * @param ry=ry paramater/y2 point for line/height - not required
	 * @param fill=colour to fill shape
	 * @param stroke=stroke colour - not required
	 * @param stroke_width= stroke thickness - not required
	 * @param info_text=text, points, path - not required
	 * @return html Doc with new reated Shape
	 *
	 */
	String createShape(String clientId, String type, String canvasId, String cx, String cy, String rx, String ry,
					   String fill, String stroke, String stroke_width, String info_text);

	/**
	 * creates Layer with layerid as name in html
	 * @param clientId String
	 * @param layerid string id
	 * @return  info about success
	 *
	 */
	String createLayer(String clientId, String layerid);

	/**
	 * registers Client, opens new Thread and returns id
	 * @return Client Id
	 */
	String registerClient();

	/**
	 * saves active Layers as one svg
	 * @param clientId String
	 * @return String in SVG format
	 *
	 */
	String getSVG(String clientId);

	/**
	 * gets shape-type and changes all parameters which are not null, example:
	 * if only cx and cy are given for a circle, it knows that circle should only be moved.
	 * @param clientId string clientid
	 * @param canvasId=Layerid
	 * @param shapeID=Name of Layer, given by user
	 * @param cx=x1-point - Default:Null
	 * @param cy=y1-point - Default:Null
	 * @param rx=radius for circle/x2 point for line - Default:Null
	 * @param ry=ry paramater/y2 point for line - Default:Null
	 * @param fill=colour - Default:Null
	 * @param stroke=stroke colour - Default:Null
	 * @param stroke_width=number of stroke thickness
	 * @param info_text=text, points, path
	 * @return html Doc of Canvas with edited Shape
	 *
	 */
	String editShape(String clientId, String canvasId, String shapeID, String cx, String cy, String rx, String ry,
					 String fill, String stroke, String stroke_width, String info_text);

	/**
	 * list with names of all created layers
	 * @param clientId string clientid
	 * @return String List with all LayerIds
	 */
	List<String> getLayers(String clientId);

	/**
	 * List with all Shapenames of Layer
	 * @param clientId string clientid
	 * @param layerId string clientid
	 * @return String List with all Shape ids of a specified Layer
	 */
	List<String> getShapesOfLayer(String clientId, String layerId);

	/**
	 * List with all possible Colours
	 * @return String List with Colours
	 */
	List<String> getColours();

	/**
	 * List with all possible Shapes
	 * @return String List with shapes. eg. rectangular, circle
	 */
	List<String> getShapes();

	/**
	 * List with all Edit options possible in menue, like changeColour, addStroke, move, delete, create,
	 * @return String List with options. e.g. create,delete etc
	 */
	List<String> getEditOptions();


	/**
	 * deletes shape with shapeid in layerID, returns exception if nothing found
	 * @param clientId String
	 * @param shapeId String
	 * @return String with message about success
	 */
	String deleteShape(String clientId, String shapeId);

	/**
	 * closes Client Data,closes and deletes saved files of Client. should be used to free resources
	 * @param ClientId string clientid
	 * @return String with message about success
	 */
	String deleteClientData(String ClientId);

	/**
	 * sets Layer active(True) or gets deactivated(false)
	 * @param clientId String
	 * @param activated boolean
	 * @param layerId String
	 * @return String with message about success
	 */
	String LayerActivition(String clientId, Boolean activated, String layerId);

	/**
	 *
	 * @param clientId string clientid
	 * @return Html Doc of Layers
	 */
	String getHtmlCanvas(String clientId);
}
